using System;
using System.Data;
using System.Web;
using System.Collections;
using System.Collections.Generic;
using System.Web.Services;
using System.Web.Services.Protocols;
using System.ComponentModel;
using MySql.Data.MySqlClient;
using System.Data.Odbc;


namespace CodeService
{
    /// <summary>
    /// Summary description for CodeWebService
    /// </summary>
    [WebService(Namespace = "http://tempuri.org/")]
    [WebServiceBinding(ConformsTo = WsiProfiles.BasicProfile1_1)]
    [ToolboxItem(false)]
    public class CodeWebService : System.Web.Services.WebService
    {
        #region Members/Properties

        //private string MyConString = "DRIVER={MySQL ODBC 3.51 Driver};" +
        //    "SERVER=software-ece.rutgers.edu;" +
        //    "DATABASE=group801;" +
        //    "UID=group801;" +
        //    "PASSWORD=ru@568;" +
        //    "OPTION=3";

        private string MyConnString = "DRIVER={MySQL ODBC 3.51 Driver};" +
            "SERVER=ee.rutgers.edu;" +
            "DATABASE=group801;" +
            "UID=group801;" +
            "PASSWORD=ru@568;" +
            "OPTION=3";

        #endregion

        #region Web Methods
        // Web Method that gets the short term prediction results from the database
        [WebMethod]
        public ShortTermPrediction GetShortTermPrediction(TickerInfo ticker)
        {
            ShortTermPrediction prediction = new ShortTermPrediction();
            string sqlQuery = "SELECT decision, simval, var FROM group801.rag_val WHERE id = " +
                ticker.Id.ToString();

            using (OdbcConnection dbConnection = new OdbcConnection(MyConnString))
            {
                dbConnection.Open();

                using (OdbcCommand cmd = new OdbcCommand(sqlQuery, dbConnection))
                {
                    using (OdbcDataReader dataReader = cmd.ExecuteReader())
                    {
                        while (dataReader.Read())
                        {
                            prediction.PredictedPrice = dataReader.GetFloat(dataReader.GetOrdinal("simval"));
                            prediction.Prediction = dataReader.GetString(dataReader.GetOrdinal("decision"));
                            prediction.ValueAtRisk = dataReader.GetFloat(dataReader.GetOrdinal("var"));
                            break; //we expect back only one row from the database.
                        }

                        if (dataReader != null && !dataReader.IsClosed)
                        {
                            dataReader.Close();
                        }
                    }
                }

                if (dbConnection != null && dbConnection.State != ConnectionState.Closed)
                {
                    dbConnection.Close();
                }
            }

            return prediction;
        }

        //We Method that gets the long term prediction results from the database
        [WebMethod]
        public LongTermPrediction GetLongTermPrediction(TickerInfo ticker)
        {
            LongTermPrediction prediction = new LongTermPrediction();
            string sqlQuery = "SELECT decision, pattern FROM group801.sur_val WHERE ID = " +
                ticker.Id.ToString();

            using (OdbcConnection dbConnection = new OdbcConnection(MyConnString))
            {
                dbConnection.Open();

                using (OdbcCommand cmd = new OdbcCommand(sqlQuery, dbConnection))
                {
                    using (OdbcDataReader dataReader = cmd.ExecuteReader())
                    {
                        while (dataReader.Read())
                        {
                            prediction.Prediction = this.GetDecisionDescription(dataReader.GetString(
                                dataReader.GetOrdinal("decision")));
                            prediction.Pattern = this.GetPatternDescription(dataReader.GetInt32(
                                dataReader.GetOrdinal("pattern")));

                            break; //we expect back only one row from the database.
                        }

                        if (dataReader != null && !dataReader.IsClosed)
                        {
                            dataReader.Close();
                        }
                    }
                }

                if (dbConnection != null && dbConnection.State != ConnectionState.Closed)
                {
                    dbConnection.Close();
                }
            }

            return prediction;
        }

        //Web Method that gets all ticker values from the database
        [WebMethod]
        public List<TickerInfo> GetAllTickers()
        {
            List<TickerInfo> data = new List<TickerInfo>();
            string sqlQuery = "SELECT num, name, id FROM group801.compid";

            using (OdbcConnection dbConnection = new OdbcConnection(MyConnString))
            {
                dbConnection.Open();

                using (OdbcCommand cmd = new OdbcCommand(sqlQuery, dbConnection))
                {
                    using (OdbcDataReader dataReader = cmd.ExecuteReader())
                    {
                        while (dataReader.Read())
                        {
                            TickerInfo tickerInfo = new TickerInfo(
                                dataReader.GetInt32(dataReader.GetOrdinal("num")),
                                dataReader.GetString(dataReader.GetOrdinal("name")),
                                dataReader.GetString(dataReader.GetOrdinal("id")));
                            data.Add(tickerInfo);
                        }

                        if (dataReader != null && !dataReader.IsClosed)
                        {
                            dataReader.Close();
                        }
                    }
                }

                if (dbConnection != null && dbConnection.State != ConnectionState.Closed)
                {
                    dbConnection.Close();
                }
            }

            return data;
        }

        //Web Method that gets stock history for a particular ticker from the database
        [WebMethod]
        public List<TickerHistory> GetHistory(TickerInfo ticker)
        {
            List<TickerHistory> data = new List<TickerHistory>();
            string sqlQuery = "SELECT id, dt, Open, high, low, close, volume, price FROM group801.historical WHERE id = '" + ticker.Symbol + 
                "' ORDER BY dt DESC";

            using (OdbcConnection dbConnection = new OdbcConnection(MyConnString))
            {
                dbConnection.Open();

                using (OdbcCommand cmd = new OdbcCommand(sqlQuery, dbConnection))
                {
                    using (OdbcDataReader dataReader = cmd.ExecuteReader())
                    {
                        while (dataReader.Read())
                        {
                            TickerHistory history = new TickerHistory();
                            history.Close = dataReader.GetDecimal(dataReader.GetOrdinal("close"));
                            history.Dt = dataReader.GetDate(dataReader.GetOrdinal("dt"));
                            history.High = dataReader.GetDecimal(dataReader.GetOrdinal("high"));
                            history.Low = dataReader.GetDecimal(dataReader.GetOrdinal("low"));
                            history.Open = dataReader.GetDecimal(dataReader.GetOrdinal("Open"));
                            history.Price = dataReader.GetDecimal(dataReader.GetOrdinal("price"));
                            history.Volume = dataReader.GetInt32(dataReader.GetOrdinal("volume"));
                            data.Add(history);
                        }

                        if (dataReader != null && !dataReader.IsClosed)
                        {
                            dataReader.Close();
                        }
                    }
                }

                if (dbConnection != null && dbConnection.State != ConnectionState.Closed)
                {
                    dbConnection.Close();
                }
            }

            return data;
        }

        #endregion

        #region Helper Methods

        private string GetPatternDescription(int patternID)
        {
            string desc = string.Empty;

            if (patternID == 1)
            {
                desc = "Cup and Saucer";
            }
            else if (patternID == 2)
            {
                desc = "Ascending Triangle";
            }
            else if (patternID == 3)
            {
                desc = "Descending Triangle";
            }
            else if (patternID == 3)
            {
                desc = "Head and Shoulders";
            }

            return desc;
        }

        private string GetDecisionDescription(string decisionID)
        {
            string desc = string.Empty;

            if (! string.IsNullOrEmpty(decisionID))
            {
                if (decisionID.Equals("1"))
                {
                    desc = "Buy";
                }
                else if (decisionID.Equals("2"))
                {
                    desc = "Sell";
                }
                else if (decisionID.Equals("3"))
                {
                    desc = "Hold";
                }
            }

            return desc;
        }

        #endregion
    }
}
